<?php


namespace app\bug\controller;


use app\bug\model\WorkAction;
use app\bug\model\WorkUser;
use app\user\model\Account;
use app\user\model\AccountCompany;use think\App;
use think\facade\Validate;
class WorkRole extends Base{
	public function __construct(App $app) {
		parent::__construct($app);
		$this->model= new \app\bug\model\WorkRole();
	}
	
	public function create(){
        $param = $this->request->param(["role_name"=>"","companyNo"=>"","work_id"=>"","action"=>[]],"post","trim");
        $valid=Validate::rule([
        	"role_name|角色名称"=>"require|max:255|unique:\app\bug\model\WorkRole,role_name^companyNo^belong",
        	"companyNo|业务公司"=>"require|max:255",
        	"work_id|岗位模板id"=>"require|number",
        	"action|角色权限集合"=>"require|array"
        	]);
        if($valid->check($param)==false) return error($valid->getError());
        $template = \app\bug\model\Work::with(["workTemplate"])->findOrEmpty($param['work_id']);
        if($template->isEmpty()) return error("岗位模板不存在");
        if($template->status==0)return error('岗位模板已禁用');
         if(empty($template->action??[]))return error('岗位模板权限不能为空');
        if(empty(array_diff($param['action'],$template->action))==false)return error('选择权限超出模板权限范围');
        $data=[
        	"role_name"=>$param['role_name'],
        	"companyNo"=>$param['companyNo'],
        	"work_id"=>$param['work_id'],
        	"belong"=>$template->belong,
        	"action"=>$param["action"],
        	"process"=>[],
        	"apply_id"=>$this->uid,
        	"apply_name"=>$this->uname,
        	];
        $add= $this->model->save($data);
        return $add ? success('岗位角色创建成功'):error('岗位角色创建失败');
	}
	
	public function save(){
        $param = $this->request->param(["id"=>"",'role_name'=>'','companyNo'=>'','work_id'=>'','action'=>[]],'post','trim');
        $valid=Validate::rule([
        	'id|岗位角色id'=>'require|number|gt:0',
        	'role_name|角色名称'=>'require|max:255|unique:\app\bug\model\WorkRole,role_name^companyNo^belong',
        	'companyNo|业务公司'=>'require|max:255',
        	'work_id|岗位模板id'=>'require|number',
        	'action|角色权限集合'=>'require|array'
        	]);
        if($valid->check($param)==false) return error($valid->getError());
        $info = $this->model->findOrEmpty($param['id']);
        if($info->isEmpty()) return error('岗位角色不存在');
        $template = \app\bug\model\Work::with(['workTemplate'])->findOrEmpty($param['work_id']);
        if($template->isEmpty()) return error('岗位模板不存在');
        if($template->status==0)return error('岗位模板已禁用');
         if(empty($template->action??[]))return error('岗位模板权限不能为空');
        if(empty(array_diff($param['action'],$template->action))==false)return error('选择权限超出模板权限范围');
        $user = WorkUser::where(['role_id'=>$param['id'],"status"=>[1,2]])->findOrEmpty();
        if($user->isEmpty()==false)return error('岗位角色有账户正在使用中无法修改');
        $info->role_name= $param['role_name'];
        $info->companyNo= $param['companyNo'];
        $info->work_id= $param['work_id'];
        $info->belong= $template->belong;
        $info->action= $param['action'];
        $add= $info->save();
        return $add ? success('岗位角色修改成功'):error('岗位角色修改失败');
	}
	
	public function  info(){
		$id = $this->request->post("id/d");
		$info = $this->model->findOrEmpty($id);
        if($info->isEmpty()) return error('岗位角色不存在');
        $info->actionInfo= (new WorkAction())->GetTreeActionByIdArr($info->action,$info->belong);
        return success('获取成功',$info);
	}
	public function  status(){
		$param = $this->request->param(['id'=>'','status'=>''],'post','trim');
		$valid = Validate::rule([
					    'id|账户角色ID'=>'require|number|gt:0',
					    'status|状态'=>'require|number|in:0,1'
					    ]);
		if($valid->check($param)==false) return error($valid->getError());
		$info = $this->model->with(['companyInfo','workInfo'])->findOrEmpty($param['id']);
		if($info->isEmpty())return error('岗位角色信息不存在');
		$info->status=$param['status'];
		$add = $info->save();
		$messg= \app\bug\model\WorkRole::$statusCn[$param['status']];
		return $add ? success("岗位角色{$messg}成功"):error("岗位角色{$messg}失败");
	}
	public function  delete(){
		$id = $this->request->post('id/d');
		$info = $this->model->findOrEmpty($id);
        if($info->isEmpty()) return error('岗位角色不存在');
        $user = WorkUser::where(["role_id"=>$id])->findOrEmpty();
        if($user->isEmpty()==false)return error('岗位角色有账户正在使用无法删除');
        $delete = $info->delete();
        return $delete ? success('删除成功'):error('删除失败');
	}
	
	public function  list(){
		$param = $this->request->param(['role_name'=>'','companyNo'=>'','belong'=>'','work_id'=>'','page'=>1,"size"=>15],'post','trim');
		$where=[];
		if($param['companyNo']!='')$where[]=['companyNo','=',$param['companyNo']];
		if($param['belong']!='')$where[]=['belong','=',$param['belong']];
		if($param['role_name']!='')$where[]=['role_name','like',"%{$param['role_name']}%"];
		if($param['work_id']!='')$where[]=['work_id','=',$param['work_id']];
		$list =$this->model->with(['companyInfo','workInfo'])->where($where)->order('id desc')->paginate(['list_rows'=>$param['size'],
		'page'=>$param['page']]);
		foreach ($list->items() as &$item){
			$item["usedNum"] = WorkUser::where(['role_id'=>$item->id])->count();
		}
		return success('获取成功',['list'=>$list->items(),'count'=>$list->total()]);
	}
	
	public function WorkRoleCreate(){
	   $param=$this->request->param(["account_id"=>"","role_id"=>""],"post","trim");
	   $valid = Validate::rule([
	   	"account_id|账户ID"=>"require|number|gt:0",
	   	"role_id|岗位角色Id"=>"require|number|gt:0"
	   	]);
	   if($valid->check($param)==false) return error($valid->getError());
	   $userinfo = Account::with(["userInfo"])->where(["is_del"=>0])->findOrEmpty($param['account_id']);
	   if($userinfo->isEmpty())return error("用户信息不存在");
	   $roleInfo = $this->model->findOrEmpty($param['role_id']);
	   if($roleInfo->isEmpty())return error('角色信息不存在');
	   $data=[
		    "account_id"=>$param['account_id'],
		    "role_id" => $param['account_id'],
		    'apply_id'=>$this->uid,
		    'apply_name'=>$this->uname,
		    ];
	   $add =WorkUser::create($data);
	   return $add ? success('账户角色设置成功'):error('账户角色设置失败');
	}
	
	public function WorkRoleSave(){
	   $param=$this->request->param(['id'=>'',"account_id"=>"",'role_id'=>''],'post','trim');
	   $valid = Validate::rule([
	   	'id|账户角色ID'=>'require|number|gt:0',
	   	'account_id|账户ID'=>'require|number|gt:0',
	   	'role_id|岗位角色Id'=>'require|number|gt:0'
	   	]);
	   if($valid->check($param)==false) return error($valid->getError());
	   $info = WorkUser::where(["id"=>$param['id']])->findOrEmpty();
	   if($info->isEmpty())return error('账户角色信息不存在');
	   if($info->status!=0)return error('账户角色状态不可修改');
	   $userinfo = Account::with(['userInfo'])->where(['is_del'=>0])->findOrEmpty($param['account_id']);
	   if($userinfo->isEmpty())return error('用户信息不存在');
	   $roleInfo = $this->model->findOrEmpty($param['role_id']);
	   if($roleInfo->isEmpty())return error('角色信息不存在');
	   $info->account_id=$param['account_id'];
	   $info->role_id=$param['role_id'];
	   $add = $info->save();
	   return $add ? success('账户角色设置成功'):error('账户角色设置失败');
	}
	
	public function WorkRoleStatus(){
	   $param=$this->request->param(['id'=>'','status'=>''],'post','trim');
	   $valid = Validate::rule([
	   	'id|账户角色ID'=>'require|number|gt:0',
	   	'status|状态'=>'require|number|in:0,1,4'
	   	]);
	   if($valid->check($param)==false) return error($valid->getError());
	   $info = WorkUser::where(['id'=>$param['id']])->findOrEmpty();
	   if($info->isEmpty())return error('账户角色信息不存在');
	   $info->status=$param['status'];
	   $add = $info->save();
	   $messg= WorkUser::$statusCn[$param['status']];
	   return $add ? success("账户角色{$messg}成功"):error("账户角色{$messg}失败");
	}
	
	public function WorkRoleInfo(){
	   $id=$this->request->post("id/d");
	   $info = WorkUser::with(['account',"workRole"])->where(['id'=>$id])->findOrEmpty();
	   if($info->isEmpty())return error('账户角色信息不存在');
	   $info->statusCn= WorkUser::$statusCn[$info->status];
	   return  success("获取成功",$info);
	}
	
	public function WorkRoleDelete(){
	   $id=$this->request->post('id/d');
	   $info = WorkUser::where(['id'=>$id])->findOrEmpty();
	   if($info->isEmpty())return error('账户角色信息不存在');
	   if($info->status==1 || $info->status==2 )return error('账户角色状态不可删除');
	   return  success('获取成功',$info);
	}
	
//	public function WorkRoleList(){
//	    $param = $this->request->param(['role_id'=>'','companyNo'=>'','status'=>'','apply_id'=>'','work_id'=>'',
//	    'belong'=>'','account_id'=>'','page'=>1,'size'=>15],'post','trim');
//		$where=[];
//		if($param['companyNo']!='')$where[]=['workRole.companyNo','=',$param['companyNo']];
//		if($param['belong']!='')$where[]=['workRole.belong','=',$param['belong']];
//		if($param['role_name']!='')$where[]=['role_id','=',$param['role_id']];
//		if($param['status']!='')$where[]=['workUser.status','=',$param['status']];
//		if($param['apply_id']!='')$where[]=['workUser.apply_id','=',$param['apply_id']];
//		$list =WorkUser::with(['account'])->withJoin(['workRole'],'left')->where($where)->order('workUser.id desc')
//		->paginate(['list_rows'=>$param['size'],'page'=>$param['page']]);
//
//		return success('获取成功',['list'=>$list->items(),'count'=>$list->total()]);
//	}
//
	public function WorkRoleList(){
	    $param = $this->request->param(['companyNo'=>'','status'=>'','apply_id'=>'',
	    'belong'=>'','account_id'=>'','page'=>1,'size'=>15],'post','trim');
		$where=[["is_del","=",0]];
		$workWhere=[['status','=',1]];
		if($param['companyNo']!='')$where[]=['companyNo','=',$param['companyNo']];
		if($param['belong']!=''){
			$workWhere[]=["belong","=",$param['belong']];
		}
		if($param['apply_id']!=''){
			$workWhere[]=['workUser.apply_id','=',$param['apply_id']];
		}
		$uidArr = WorkUser::withJoin(['workRole'],"left")->where($workWhere)->column('account_id');
		if($param['status']!==''){
			if($param['status']==1){
				$where[]=['account_id','in',$uidArr];
			}else{
				$where[]=['account_id','not in',$uidArr];
			}
		}
		
	
		$list =AccountCompany::with(["accountInfo"=>["userInfo","accountItem"=>["itemName"]]])->where($where)->order('id asc')
		->paginate(['list_rows'=>$param['size'],'page'=>$param['page']]);
//		$uidArr = WorkUser::with(['workRole'])->where(['account_id'=>array_unique(array_column($list->items(),"account_id"))])
//		->column("role_id,status,apply_name",'account_id');
		foreach ($list->items() as &$item){
		$uidArr = WorkUser::withJoin(['workRole'],'left')
		->where(['account_id'=>$item['account_id'],'companyNo'=>$item['companyNo'],"belong"=>$param['belong']])
		->field("work_user.*,workRole.role_name")
		->findOrEmpty();
				$item['role_name'] ='';
				$item['status'] =0;
				$item['apply_name'] = '';
				$item['usedNum']=0;
			if($uidArr->isEmpty()==false){
				$item['role_name'] = $uidArr->role_name??'';
				$item['status'] = $uidArr->status??0;
				$item['apply_name'] = $uidArr->apply_name??'';
				$item['usedNum'] = WorkUser::where(['role_id'=>$uidArr->role_id])->count();
			}
		}
		return success('获取成功',['list'=>$list->items(),'count'=>$list->total()]);
	}
}